/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "emacs.h"
#include "templates/templates.h"
#include "functions.h"



namespace Fun
{

Emacs::Emacs()
{
	fun.url = "emacs";
}



bool Emacs::HasAccess(const Item & item)
{
	if( request->session->puser && request->session->puser->super_user )
		// super user can use emacs everywhere
		return true;

	if( !system->HasWriteAccess(item) )
		return false;

	if( !system->mounts.pmount->IsPar(system->mounts.MountParEmacsOn()) )
		return true;

	if( system->mounts.pmount->IsArg(system->mounts.MountParEmacsOn(), request->dir_tab.size()) )
		return true;	
	
return false;
}



bool Emacs::HasAccess()
{
	return HasAccess(*request->last_item);
}





bool Emacs::PostEmacsCheckAbuse(bool adding)
{
	if( !system->rebus.CheckRebus() )
	{
		request->status = WINIX_ERR_INCORRECT_REBUS;
		return false;
	}

	// !! is tested in createthread once
	functions->CheckGetPostTimes();

	if( request->session->spam_score > 0 )
	{
		request->status = WINIX_ERR_SPAM;
		log << log1 << "Emacs: ignoring due to suspected spamming" << logend;
		return false;
	}

return true;
}



void Emacs::PostFunEmacsModifyMountPoint(bool adding)
{
	if( system->mounts.pmount->type == system->mounts.MountTypeThread() )
	{
		if( adding )
			db->EditThreadAddItem(request->dir_tab.back()->id, request->item.id);

		system->RedirectToLastDir();
	}
	/* !! tticket
	else
	if( system->mounts.pmount->type == system->mounts.MountTypeTicket() )
	{
		system->RedirectToLastDir();
	}
	*/
	else
	{
		// system->mounts.MountTypeCms()
		system->RedirectTo(request->item);
	}
}



void Emacs::MakePost()
{
	bool adding        = !request->is_item;
	bool edit_with_url = functions->ReadItem(request->item, Item::file);

	if( adding )
		functions->SetUser(request->item); // set user before checking the rebus

	if( !PostEmacsCheckAbuse(adding) )
		return;


	if( adding )
	{
		request->is_item          = true;
		request->item.privileges  = 0644; // !! tymczasowo, bedzie uzyte umask
		request->status = system->AddFile(request->item);
	}
	else
	{
		// !! moze dodac metode EditFile w klasie functions? 
		// i ona wywola system->EditFile i poniszego updatera
		request->status = system->EditFile(request->item, edit_with_url);

		if( request->status == WINIX_ERR_OK )
			TemplatesFunctions::pattern_cacher.UpdatePattern(request->item);
	}


	if( request->status == WINIX_ERR_OK )
	{
		PostFunEmacsModifyMountPoint(adding);
		functions->CheckSpecialFile(request->item);
	}
	else
	{
		log << log1 << "Emacs: error: "
			<< request->status << logend;
	}
}








} // namespace
