/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "editticket.h"
#include "readticket.h"
#include "functions/functions.h"




namespace Ticket
{

EditTicket::EditTicket()
{
	fun.url = "editticket";
}


void EditTicket::SetTDb(TDb * ptdb)
{
	tdb = ptdb;
}


void EditTicket::SetTicketInfo(TicketInfo * pinfo)
{
	ticket_info = pinfo;
}


void EditTicket::Clear()
{
	ticket_info->Clear();
}


bool EditTicket::HasAccess()
{
	// not logged users cannot edit tickets
	if( !request->session->puser )
		return false;

	if( request->dir_tab.empty() )
		return false;
	
	ticket_info->ReadTicket(request->dir_tab.back()->id);

	if( request->is_item || !ticket_info->is_ticket )
		return false;
	
	if( !system->HasWriteAccess(*request->dir_tab.back()) )
		return false;
	
	if( !system->mounts.pmount || system->mounts.pmount->type != ticket_info->mount_type_ticket )
		return false;

return true;
}





void EditTicket::EditTicketCheckFirstItem()
{
	if( request->status != WINIX_ERR_OK )
		return;


	if( ticket_info->ticket.item_id == -1 )
	{
		// creating a new item (the item was deleted by a user)

		Item item;
		item.parent_id  = request->dir_tab.back()->id;
		item.subject    = request->dir_tab.back()->subject;
		item.type       = Item::file;
		item.privileges = 0644; // !! tymczasowo
		functions->SetUser(item);
		functions->PrepareUrl(item);

		request->status = db->AddItem(item); // !! moze system->AddFile() tutaj?

		if( request->status == WINIX_ERR_OK )
		{
			log << log2 << "EditTicket: added the first item with content for the ticket, item.id: " << item.id << logend;
			ticket_info->ticket.item_id = item.id;
		}
	}
}



void EditTicket::EditTicketModTicket()
{
	if( request->status != WINIX_ERR_OK )
		return;

	Ticket ticket;
	ReadTicket read_ticket;
	read_ticket.Read(system, request, ticket_info, &ticket);

	ticket_info->ticket.type     = ticket.type;
	ticket_info->ticket.status   = ticket.status;
	ticket_info->ticket.priority = ticket.priority;
	ticket_info->ticket.category = ticket.category;
	ticket_info->ticket.expected = ticket.expected;
	ticket_info->ticket.progress = ticket.progress;

	request->status = tdb->EditTicketById(ticket_info->ticket);
}



void EditTicket::EditTicketModDir()
{
	if( request->status != WINIX_ERR_OK )
		return;

	// we don't modify the url

	Item & dir = *request->dir_tab.back();
	bool edit_dir_subject = (dir.subject != request->item.subject);

	if( edit_dir_subject )
	{
		dir.subject = request->item.subject;
		request->status = db->EditItemById(dir, false);
	}
}


void EditTicket::EditTicketModFirstItem()
{
	if( request->status != WINIX_ERR_OK )
		return;

	// modyfing the first item (the one with content)
	functions->ReadItemContentWithType(request->item);

	Item item;
	db->GetItemById(ticket_info->ticket.item_id, item);
	item.subject      = request->item.subject;
	item.content      = request->item.content;
	item.content_type = request->item.content_type;
	// the url doesn't matter

	db->EditItemById(item, false);
}



void EditTicket::PostFunEditTicketLogAndRedirect()
{
	if( request->status == WINIX_ERR_OK )
	{
		log << log2 << "Content: edited a ticket" << logend;
		system->RedirectToLastDir();
	}
	else
	{
		log << log1 << "Content: problem with editing a ticket, error code: "
			<< request->status << logend;
	}
}


bool EditTicket::FunCreateTicketCheckAbuse()
{
	if( !system->rebus.CheckRebus() )
	{
		request->status = WINIX_ERR_INCORRECT_REBUS;
		return false;
	}

	functions->CheckGetPostTimes();

	if( request->session->spam_score > 0 )
	{
		request->status = WINIX_ERR_SPAM;
		log << log1 << "Content: ignoring due to suspected spamming" << logend;
		return false;
	}

return true;
}


void EditTicket::MakePost()
{
	functions->ReadItem(request->item, Item::dir);
	// co z uzytkownikiem i grup? chyba nie wczytane? !!! (wczesniej bylo SetUser() w ReadItem() - teraz nie ma)
	// dodac informacje o modyfikacji !!!

	if( !FunCreateTicketCheckAbuse() )
	{
		functions->ReadItemContentWithType(request->item); // for correctly displaying the form
		return;
	}

	if( tdb->GetTicketByDirId(request->dir_tab.back()->id, ticket_info->ticket) != WINIX_ERR_OK )
	{
		request->status = WINIX_ERR_PERMISSION_DENIED;
		return;
	}

	EditTicketCheckFirstItem();
	EditTicketModTicket();
	EditTicketModDir();
	EditTicketModFirstItem();
	PostFunEditTicketLogAndRedirect();
}



void EditTicket::MakeGet()
{
	db->GetItemById(ticket_info->ticket.item_id, request->item);

	// the subject we get from the last directory
	request->item.subject = request->dir_tab.back()->subject;
}



} // namespace
