/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "tdb.h"
#include "funticket.h"
#include "editticket.h"
#include "createticket.h"
#include "core/log.h"
#include "core/plugin.h"


extern "C" void Init(PluginInfo &);


namespace Ticket
{
const char plugin_name[] = "ticket";

TDb tdb;
TicketInfo ticket_info;

FunTicket    fun_ticket;
CreateTicket fun_create_ticket;
EditTicket   fun_edit_ticket;



void AddFunctions(PluginInfo & info)
{
	info.functions->Add(fun_ticket);
	info.functions->Add(fun_create_ticket);
	info.functions->Add(fun_edit_ticket);
}


void SelectDefaultFunction(PluginInfo & info)
{
	if( info.system->mounts.pmount->type == ticket_info.mount_type_ticket )
		info.request->function = &fun_ticket;
}


void AddMounts(PluginInfo & info)
{
	Mounts & m = info.system->mounts;
	
	ticket_info.mount_type_ticket					= m.AddMountType("ticket");
	ticket_info.mount_par_ticket					= m.AddMountPar("ticket");
	ticket_info.mount_par_ticket_type				= m.AddMountPar("ticket_type");
	ticket_info.mount_par_ticket_type_default		= m.AddMountPar("ticket_type_default");
	ticket_info.mount_par_ticket_status				= m.AddMountPar("ticket_status");
	ticket_info.mount_par_ticket_status_default		= m.AddMountPar("ticket_status_default");
	ticket_info.mount_par_ticket_priority			= m.AddMountPar("ticket_priority");
	ticket_info.mount_par_ticket_priority_default	= m.AddMountPar("ticket_priority_default");
	ticket_info.mount_par_ticket_category			= m.AddMountPar("ticket_category");
	ticket_info.mount_par_ticket_category_default	= m.AddMountPar("ticket_category_default");
	ticket_info.mount_par_ticket_expected			= m.AddMountPar("ticket_expected");
	ticket_info.mount_par_ticket_expected_default	= m.AddMountPar("ticket_expected_default");
	ticket_info.mount_par_createticket_on			= m.AddMountPar("createticket_on");
}


void AddEzcFunctions(PluginInfo & info);

} // namespace Ticket



void Init(PluginInfo & info)
{
using namespace Ticket;

	plugin.Assign(WINIX_TEMPLATES_CREATEFUNCTIONS,	AddEzcFunctions);
	plugin.Assign(WINIX_CREATE_FUNCTIONS,			AddFunctions);
	plugin.Assign(WINIX_SELECT_DEFAULT_FUNCTION,	SelectDefaultFunction);
	plugin.Assign(WINIX_ADD_MOUNTS,					AddMounts);

	tdb.SetConn(info.db->GetConn());

	ticket_info.SetDb(info.db);
	ticket_info.SetTDb(&tdb);

	fun_ticket.SetTDb(&tdb);
	fun_ticket.SetTicketInfo(&ticket_info);

	fun_edit_ticket.SetTDb(&tdb);
	fun_edit_ticket.SetTicketInfo(&ticket_info);

	fun_create_ticket.SetTDb(&tdb);
	fun_create_ticket.SetTicketInfo(&ticket_info);

	info.p1 = (void*)plugin_name;
}


